using Toybox.Activity as Act;
using Toybox.ActivityMonitor as Amon;
using Toybox.Graphics as Gfx;
using Toybox.Lang;
using Toybox.Math;
using Toybox.System as Sys;
using Toybox.Time.Gregorian as Cal;
using Toybox.WatchUi as Ui;

using C as C;

class FacadeView extends Ui.WatchFace
{

//
//  Classes to handle the hands
//
private var hr_hand;
private var min_hand;
private var sec_hand;
private var hr2_hand;

//
//  Classes for hash marks around the dial
//
private var big_hash;
private var sml_hash;

//
//  Class for the moon phase calculator
//
private var moon;

//
//  Day abbreviations (3 characters from the standard API is too wide
//
private var days = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"];

function initialize()
{
    C.logger("initialize()");
    WatchFace.initialize();
}

// Load your resources here
function onLayout(dc)
{

    C.logger("onLayout()");
    C.center_x = dc.getWidth() / 2;
    C.center_y = dc.getHeight() / 2;
    C.radius = (C.center_x > C.center_y) ? C.center_y : C.center_x;
    C.logger("radius:" + C.radius);

    var r10 = C.radius * 0.1;
    big_hash = new Hash(C.radius, r10, 6, C.WHITE);
    sml_hash = new Hash(C.radius, r10/2, 2, C.WHITE);

    hr_hand = new Hand(C.radius * 0.6, r10/2, C.alpha(C.WHITE, 0xe0));
    min_hand = new Hand(C.radius * 0.95, r10/2, C.alpha(C.WHITE, 0xe0));
    sec_hand = new Hand(C.radius * 0.95, 2, C.alpha(C.RED, 0xff));
    hr2_hand = new HandHr2(C.radius, r10);

    moon = new Moon();
}

// Called when this View is brought to the foreground. Restore
// the state of this View and prepare it to be shown. This includes
// loading resources into memory.
function onShow() as Void
{

    C.logger("onShow()");
    C.settings();
}

// Update the view
function onUpdate(dc)
{
    var hr, hr2;

    //C.logger("onUpdate()");

    // Get and show the current time
    var t = Time.now();
    var now = Cal.info(t, Time.FORMAT_SHORT);
    var now2 = Cal.info(t, Time.FORMAT_SHORT);
    C.tz_adj(now, C.tz1_info);
    C.tz_adj(now2, C.tz2_info);

    hr = hr_12(now, C.tz1_info);
    hr2 = hr_12(now2, C.tz2_info);

    dc.setColor(C.WHITE, C.BLACK);
    dc.clear();
    dc.setColor(C.WHITE, C.TRANSPARENT);

    sh_dial(dc);

    sh_days(dc, now);

    sh_battery(dc);

    sh_hr(dc);

    sh_steps(dc);

    moon.show(dc);

    hr2_hand.show(dc, hr2);
    dc.setColor(C.WHITE, C.TRANSPARENT);
    hr_hand.show(dc, hr[0], 12 * 60, ((C.tz1 != 0) ? hr[1] : null));
    sh_ampm(dc, hr[2]);
    min_hand.show(dc, now.min, 60, null);
    sec_hand.show(dc, now.sec, 60, null);

    dc.setColor(C.WHITE, C.TRANSPARENT);
    dc.fillCircle(C.center_x, C.center_y, C.dot_sz);
    //dc.drawArc(C.center_x, C.center_y, 100, Gfx.ARC_CLOCKWISE, 0, 90);
}

// Called when this View is removed from the screen. Save the
// state of this View here. This includes freeing resources from
// memory.
function onHide() as Void
{

    C.logger("onHide()");
}

// The user has just looked at their watch. Timers and animations may be started here.
function onExitSleep() as Void
{

    C.logger("onExitSleep()");
}

// Terminate any active timers and prepare for slow updates.
function onEnterSleep() as Void
{

    C.logger("onEnterSleep()");
}

function hr_12(now, info)
{
    var hr;

    if (now.hour >= 12) {
        hr = [ now.hour - 12, info[1], true ];
    } else {
        hr = [ now.hour, info[1], false ];
    }
    hr[0] = (hr[0] * 60) + now.min;
    return hr;
}

function sh_dial(dc)
{
    for (var i = 0; i < 60; i += 1) {
        if ((i % 5) == 0) {
            big_hash.show(dc, i, 60);
        } else {
            sml_hash.show(dc, i, 60);
        }
    }
    var id = big_hash.start_x();
    dc.drawText(C.center_x, C.center_y - id, Gfx.FONT_XTINY, "12", Gfx.TEXT_JUSTIFY_CENTER);
}

function sh_ampm(dc, pm)
{

    switch (C.ampm) {

    case 0:             // no indicator
        return;

    case 2:             // AM or PM as appropriate
        if (!pm) {
            dc.setColor(C.WHITE, C.TRANSPARENT);
            dc.drawText(C.center_x, C.center_y + C.dot_sz, Gfx.FONT_XTINY, "AM", Gfx.TEXT_JUSTIFY_CENTER);
            return;
        }
        // fall thru
    case 1:             // PM only
        if (pm) {
            dc.setColor(C.WHITE, C.TRANSPARENT);
            dc.drawText(C.center_x, C.center_y + C.dot_sz, Gfx.FONT_XTINY, "PM", Gfx.TEXT_JUSTIFY_CENTER);
        }
        break;

    }
}

function sh_days(dc, today)
{

    var dw = days[today.day_of_week - 1];
    dc.setColor(C.WHITE, C.TRANSPARENT);
    dc.drawText(C.center_x * 0.3, C.center_y, Gfx.FONT_TINY, dw, Gfx.TEXT_JUSTIFY_LEFT | Gfx.TEXT_JUSTIFY_VCENTER);
    var x = C.center_x + (C.center_x * 0.7);
    var y = C.center_y;
    dc.drawText(C.center_x + (C.center_x * 0.7), C.center_y, Gfx.FONT_TINY, today.day, Gfx.TEXT_JUSTIFY_RIGHT | Gfx.TEXT_JUSTIFY_VCENTER);
    var dim = dc.getTextDimensions("88", Gfx.FONT_TINY);
    var w = dim[0];
    var h = dim[1];
    dc.setPenWidth(1);
    dc.drawRectangle(x - w - 2, y - h/2, w + 6, h);
}

function sh_battery(dc)
{
    var color, icon;

    var lvl = Sys.getSystemStats().battery.toNumber();
    if (lvl >= 50) {
        color = C.GREEN;
        icon = Rez.Drawables.battery100;
    } else if (lvl >= 25) {
        color = C.YELLOW;
        icon = Rez.Drawables.battery050;
    } else {
        color = C.RED;
        icon = Rez.Drawables.battery000;
    }
    var x = C.center_x * 0.5;
    var y = C.center_y * 0.5;
    dc.setColor(color, C.TRANSPARENT);
    dc.drawText(x, y, Gfx.FONT_TINY, lvl, Gfx.TEXT_JUSTIFY_CENTER);

    var dim = dc.getTextDimensions("100", Gfx.FONT_TINY);
    var w = dim[0];
    var h = dim[1];
    var r = Math.sqrt(w*w + h*h) / 2;
    var a = (360 * lvl) / 100;
    a = 360 - a + 90;

    dc.setPenWidth(6);
    dc.setColor(color, C.TRANSPARENT);
    dc.drawArc(x, y + (h/2), r, Gfx.ARC_CLOCKWISE, 90, a);

    var bits = new Ui.Bitmap({ :rezId => icon,
                               :locX => x + h/2 + 10,
                               :locY => y + h });
    bits.draw(dc);
}

function sh_hr(dc)
{

    var hr = Act.getActivityInfo().currentHeartRate;
    if (hr == null) {
        hr = "--";
    }

    var x = C.center_x + (C.center_x * 0.65);
    var y = C.center_y * 0.4;
    dc.setColor(C.WHITE, C.TRANSPARENT);
    dc.drawText(x, y, Gfx.FONT_TINY, hr, Gfx.TEXT_JUSTIFY_RIGHT);

    var h = dc.getFontHeight(Gfx.FONT_TINY);
    var bits = new Ui.Bitmap({ :rezId => Rez.Drawables.heart,
                               :locX => x - 24,
                               :locY => y + h });
    bits.draw(dc);
}

function sh_steps(dc)
{

    var steps = Amon.getInfo().steps;
    if (steps == null) {
        steps = "--";
    } else {
        steps = C.n2comma(steps);
    }

    var x = C.center_x * 0.3;
    var y = C.center_y + (C.center_y * 0.3);
    dc.setColor(C.WHITE, C.TRANSPARENT);
    dc.drawText(x, y, Gfx.FONT_TINY, steps, Gfx.TEXT_JUSTIFY_LEFT);

    x = C.center_x * 0.4;
    var h = dc.getFontHeight(Gfx.FONT_TINY);
    var bits = new Ui.Bitmap({ :rezId => Rez.Drawables.steps,
                               :locX => x,
                               :locY => y + h });
    bits.draw(dc);
}

}
